/*
Copyright (c) 2006 NoisyB


This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xmlwriter.h>
#include "xml.h"


/*
Introduction

libxml2 is a library of functions for handling XML data.


A simple example:

#include <stdio.h>
#include <libxml/tree.h>


int
main ()
{
  xmlDocPtr doc;
  xmlNodePtr nodeLevel1;
  xmlNodePtr nodeLevel2;

  doc = xmlParseFile ("xmlfile.xml");
  for (nodeLevel1 = doc->children;
       nodeLevel1 != NULL; nodeLevel1 = nodeLevel1->next)
    {
      printf ("%s\n", nodeLevel1->name);
      for (nodeLevel2 = nodeLevel1->children;
           nodeLevel2 != NULL; nodeLevel2 = nodeLevel2->next)
        {
          printf ("\t%s\n", nodeLevel2->name);
        }
    }
  xmlSaveFile ("xmlfile_copy.xml", doc);
  xmlFreeDoc (doc);
  return 0;
}


The above code, compiled with -lxml2, should print out the names of the
elements in the first two elements' depth of an XML file, and save a copy
of the file.


Explanation of Introduction

The xmlDocPtr is a pointer to an xmlDoc structure.
It represents an XML data source.

You load an XML file with the xmlParseFile() function, which takes
as a parameter the name of an XML file and returns a pointer to a
new xmlDoc structure (or NULL on failure). When done, you release
this memory with the xmlFreeDoc() function. You can export an xmlDocPtr's
data as an XML file with the xmlSaveFile() function.



The xmlNodePtr points to a single element or node of an XML document.
Each xmlNode has a .children member which is an xmlNodePtr to the first
of this node's children. Each xmlNode has a .name member which is a
string containing the name of the element it represents, or the word "text"
for a text node.

The xmlNodePtr is the basic structure used to traverse an XML document
with libxml2. It contains several xmlNodePtrs which can be used to move
around the document. If there is no other node in a particular direction,
the pointer is NULL.

xmlNodePtr->children	The first child of the node
xmlNodePtr->last	The node's last child
xmlNodePtr->parent	The current node's parent node

xmlNodePtr->next	The next sibling node
xmlNodePtr->prev	The previous sibling node

xmlNodePtr->doc		The xmlDocPtr for the document containing this node


<node>
        <node> ->parent
                <node>
                        <node> ->prev
                        </node>
                        <node>
                        </node>
                </node>
                <node> You Are Here
                        <node> ->children
                        </node>
                        <node>
                        </node>
                        <node> ->last
                        </node>
                </node>
                <node> ->next
                        <node>
                        </node>
                </node>
        </node>
</node>



Although the above diagram suggests that nodes are simply elements,
be aware that areas of whitespace between elements are also nodes.
This makes the above example faulty if it is taken literally, because
the child and neighbour node pointers would point to the whitespace
between elements.
*/


xml_doc_t *
xml_parse (const char *fname)
{
  return xmlParseFile (fname);
}


xml_node_t *
xml_get_rootnode (xml_doc_t *d)
{
  if (d)
    return xmlDocGetRootElement (d);
  return NULL;
}


xml_node_t *
xml_get_childnode (xml_node_t *n)
{
  if (n)
    return n->xmlChildrenNode;
  return NULL;
}


xml_node_t *
xml_get_nextnode (xml_node_t *n)
{
  if (n)
    return n->next;
  return NULL;
}


int
xml_is_empty_node (xml_node_t *n)
{
  if (n)
    return xmlIsBlankNode (n);
  return 0;
}


const unsigned char *
xml_get_string (xml_node_t *n)
{
  if (n)
    return xmlNodeListGetString (n->doc, n, 1);
  return NULL;
}


const unsigned char *
xml_get_prop (xml_node_t *n, const char *name)
{
  if (n)
    if (xmlHasProp (n, (const unsigned char *) name))
      return xmlGetProp (n, (const unsigned char *) name);
  return NULL;
}


int
xml_strcmp (const unsigned char *a, const char *b)
{
  return strcmp ((const char *) a, b);
}


int
xml_free (xml_doc_t *d)
{
  (void) d;
  return 0;
}
